"use strict";
var binding_count_1 = require("../bindings/binding_count");
var binding_1 = require("../bindings/binding");
var lookup_1 = require("./lookup");
var planner_1 = require("../planning/planner");
var resolver_1 = require("../resolution/resolver");
var ERROR_MSGS = require("../constants/error_msgs");
var binding_to_syntax_1 = require("../syntax/binding_to_syntax");
var Kernel = (function () {
    function Kernel(options) {
        var _this = this;
        if (options === void 0) { options = { middleware: [], modules: [] }; }
        this._planner = new planner_1.default();
        this._resolver = new resolver_1.default(options.middleware);
        this._bindingDictionary = new lookup_1.default();
        if (Array.isArray(options.modules) === true) {
            options.modules.forEach(function (module) { module(_this); });
        }
    }
    Kernel.prototype.bind = function (runtimeIdentifier) {
        var binding = new binding_1.default(runtimeIdentifier);
        this._bindingDictionary.add(runtimeIdentifier, binding);
        return new binding_to_syntax_1.default(binding);
    };
    Kernel.prototype.unbind = function (runtimeIdentifier) {
        try {
            this._bindingDictionary.remove(runtimeIdentifier);
        }
        catch (e) {
            throw new Error(ERROR_MSGS.CANNOT_UNBIND + " " + runtimeIdentifier);
        }
    };
    Kernel.prototype.unbindAll = function () {
        this._bindingDictionary = new lookup_1.default();
    };
    Kernel.prototype.get = function (runtimeIdentifier) {
        var bindings = this._planner.getBindings(this, runtimeIdentifier);
        switch (bindings.length) {
            case binding_count_1.default.NoBindingsAvailable:
                throw new Error(ERROR_MSGS.NOT_REGISTERED + " " + runtimeIdentifier);
            case binding_count_1.default.OnlyOneBindingAvailable:
                return this._planAndResolve(bindings[0]);
            case binding_count_1.default.MultipleBindingsAvailable:
            default:
                throw new Error(ERROR_MSGS.AMBIGUOUS_MATCH + " " + runtimeIdentifier);
        }
    };
    Kernel.prototype.getAll = function (runtimeIdentifier) {
        var _this = this;
        var bindings = this._planner.getBindings(this, runtimeIdentifier);
        switch (bindings.length) {
            case binding_count_1.default.NoBindingsAvailable:
                throw new Error(ERROR_MSGS.NOT_REGISTERED + " " + runtimeIdentifier);
            case binding_count_1.default.OnlyOneBindingAvailable:
            case binding_count_1.default.MultipleBindingsAvailable:
            default:
                return bindings.map(function (binding) {
                    return _this._planAndResolve(binding);
                });
        }
    };
    Kernel.prototype._planAndResolve = function (binding) {
        var context = this._planner.createContext(this);
        this._planner.createPlan(context, binding);
        return this._resolver.resolve(context);
    };
    return Kernel;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = Kernel;
