"use strict";
var plan_1 = require("./plan");
var context_1 = require("./context");
var request_1 = require("./request");
var target_1 = require("./target");
var METADATA_KEY = require("../constants/metadata_keys");
var ERROR_MSGS = require("../constants/error_msgs");
var binding_type_1 = require("../bindings/binding_type");
var Planner = (function () {
    function Planner() {
    }
    Planner.prototype.createContext = function (kernel) {
        return new context_1.default(kernel);
    };
    Planner.prototype.createPlan = function (context, binding) {
        var _this = this;
        var rootRequest = new request_1.default(binding.runtimeIdentifier, context, null, binding);
        rootRequest.bindings.push(binding);
        var plan = new plan_1.default(context, rootRequest);
        context.addPlan(plan);
        var dependencies = this._getDependencies(binding.implementationType);
        dependencies.forEach(function (d) { _this._createSubRequest(rootRequest, d); });
        return plan;
    };
    Planner.prototype.getBindings = function (kernel, service) {
        var bindings = [];
        var _kernel = kernel;
        var _bindingDictionary = _kernel._bindingDictionary;
        if (_bindingDictionary.hasKey(service)) {
            bindings = _bindingDictionary.get(service);
        }
        return bindings;
    };
    Planner.prototype._createSubRequest = function (parentRequest, target) {
        var _this = this;
        try {
            var bindings = this.getBindings(parentRequest.parentContext.kernel, target.service.value());
            if (bindings.length > 1) {
                throw new Error(ERROR_MSGS.AMBIGUOUS_MATCH + " " + target.service.value());
            }
            else {
                var binding = bindings[0];
                var childRequest_1 = parentRequest.addChildRequest(target.service.value(), binding, target);
                if (binding.type === binding_type_1.default.Instance) {
                    var subDependencies = this._getDependencies(binding.implementationType);
                    subDependencies.forEach(function (d, index) {
                        _this._createSubRequest(childRequest_1, d);
                    });
                }
            }
        }
        catch (error) {
            if (error instanceof RangeError) {
                this._throwWhenCircularDependenciesFound(parentRequest.parentContext.plan.rootRequest);
            }
            else {
                throw new Error(error.message);
            }
        }
    };
    Planner.prototype._throwWhenCircularDependenciesFound = function (request, previousServices) {
        var _this = this;
        if (previousServices === void 0) { previousServices = []; }
        previousServices.push(request.service);
        request.childRequests.forEach(function (childRequest) {
            var service = childRequest.service;
            if (previousServices.indexOf(service) === -1) {
                if (childRequest.childRequests.length > 0) {
                    _this._throwWhenCircularDependenciesFound(childRequest, previousServices);
                }
                else {
                    previousServices.push(service);
                }
            }
            else {
                throw new Error(ERROR_MSGS.CIRCULAR_DEPENDENCY + " " + service + " and " + request.service);
            }
        });
    };
    Planner.prototype._getDependencies = function (func) {
        var injections = Reflect.getMetadata(METADATA_KEY.INJECT, func) || [];
        var paramNames = Reflect.getMetadata(METADATA_KEY.PARAM_NAMES, func) || [];
        var tags = Reflect.getMetadata(METADATA_KEY.TAGGED, func) || [];
        return injections.map(function (inject, index) {
            var targetName = paramNames[index];
            var target = new target_1.default(targetName, inject);
            target.metadata = tags[index.toString()] || [];
            return target;
        });
    };
    return Planner;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = Planner;
