"use strict";
var binding_scope_1 = require("../bindings/binding_scope");
var binding_type_1 = require("../bindings/binding_type");
var ERROR_MSGS = require("../constants/error_msgs");
var Resolver = (function () {
    function Resolver(middleWare) {
        if (middleWare === void 0) { middleWare = []; }
        this._middleWare = middleWare;
    }
    Resolver.prototype.resolve = function (context) {
        var rootRequest = context.plan.rootRequest;
        return this._inject(rootRequest);
    };
    Resolver.prototype._inject = function (request) {
        var _this = this;
        var childRequests = request.childRequests;
        var binding = request.bindings[0];
        switch (binding.type) {
            case binding_type_1.default.Value:
                return binding.cache;
            case binding_type_1.default.Constructor:
                return binding.implementationType;
            case binding_type_1.default.Factory:
                return binding.factory(request.parentContext);
            case binding_type_1.default.Provider:
                return binding.provider(request.parentContext);
            case binding_type_1.default.Instance:
                var constr = binding.implementationType;
                var isSingleton = binding.scope === binding_scope_1.default.Singleton;
                if (isSingleton && binding.cache !== null) {
                    return binding.cache;
                }
                if (childRequests.length > 0) {
                    var injections = childRequests.map(function (childRequest) {
                        return _this._inject(childRequest);
                    });
                    var instance = this._createInstance(constr, injections);
                    if (isSingleton) {
                        binding.cache = instance;
                    }
                    return instance;
                }
                else {
                    var instance = new constr();
                    if (isSingleton) {
                        binding.cache = instance;
                    }
                    return instance;
                }
            case binding_type_1.default.Invalid:
            default:
                throw new Error(ERROR_MSGS.INVALID_BINDING_TYPE + " " + request.service);
        }
    };
    Resolver.prototype._createInstance = function (Func, injections) {
        return new (Func.bind.apply(Func, [void 0].concat(injections)))();
    };
    return Resolver;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = Resolver;
